#!/bin/bash
#Automatically Created by buildpkg 1.6.1
#Maintainer: Christopher Rey P. Baluyut <wakizaki@gmail.com>
#url: http://www.truecrypt.org

######Begin Redundant Code######################################
check_for_root() {
	if [ "$UID" != "0" ]; then
		echo "You need to be root"
		exit 1
	fi
}

clean_dirs () {
        for COMPLETED in src pkg; do
                if [ -e $COMPLETED ]; then
                        rm -rf $COMPLETED
                fi
        done
}

clean_old_builds () {
	for COMPLETED in $package.{txz,md5,dep}; do
		if [ "$KEEPDEPFILE" = "1" ] && [ $COMPLETED = $package.dep ]; then
			continue
		elif [ -e $COMPLETED ]; then
			rm -rf $COMPLETED
		fi
	done
	clean_dirs
}

set_pre_permissions() {
	cd $startdir/src
	find . -perm 664 -exec chmod 644 {} \;
	find . -perm 600 -exec chmod 644 {} \;
	find . -perm 444 -exec chmod 644 {} \;
	find . -perm 400 -exec chmod 644 {} \;
	find . -perm 440 -exec chmod 644 {} \;
	find . -perm 777 -exec chmod 755 {} \;
	find . -perm 775 -exec chmod 755 {} \;
	find . -perm 511 -exec chmod 755 {} \;
	find . -perm 711 -exec chmod 755 {} \;
	find . -perm 555 -exec chmod 755 {} \;
}


check_for_menu_compliance() {
	if [ -d "$startdir/pkg/usr/share/applications" ]; then
		cd $startdir/pkg/usr/share/applications
		for desktop in *.desktop; do
			Icon=$(grep -e "^Icon=" $desktop | sed 's/Icon=//')
			if [[ "$Icon" ]]; then
				if [ ! "$Icon" == "${Icon%.*}" ]; then
					sed '/Icon/s/\..*$//' -i $desktop
					Icon=$(grep -e "^Icon=" $desktop | sed 's/Icon=//')
				fi
				if [ -d "$startdir/pkg/usr/share/pixmaps" ]; then
					cd $startdir/pkg/usr/share/pixmaps
					for pixmaps in *; do
						[ -d $pixmaps ] && continue
						[ ! "${pixmaps##*.}" == "png" -a ! "${pixmaps##*.}" == "svg" ] && continue
				 		if [ "${pixmaps%.*}" == "$Icon" ]; then
				 			DIR=$(file $pixmaps | awk -F, '{print $2}' | sed 's/ //g')
							[ o"$DIR" == "o" ] && DIR="scalable"
							mkdir -p $startdir/pkg/usr/share/icons/hicolor/$DIR/apps
							mv $pixmaps $startdir/pkg/usr/share/icons/hicolor/$DIR/apps
							if [[ ! "$(ls $startdir/pkg/usr/share/pixmaps)" ]]; then
								rm -rf $startdir/pkg/usr/share/pixmaps
							fi
						fi
					done
				fi
				if [ -d "$startdir/pkg/usr/share/icons" ]; then
					cd $startdir/pkg/usr/share/icons
					for icons in *; do
				 		if [ "${icons%.*}" == "$Icon" ]; then
							[ ! "${pixmaps##*.}" == "png" -a ! "${pixmaps##*.}" == "svg" ] && continue
				 			DIR=$(file $icons | awk -F, '{print $2}' | sed 's/ //g')
							[ o"$DIR" == "o" ] && DIR="scalable"
							mkdir -p $startdir/pkg/usr/share/icons/hicolor/$DIR/apps
							mv $icons $startdir/pkg/usr/share/icons/hicolor/$DIR/apps
						fi
					done
				fi
			fi
		done
	fi
}


gzip_man_and_info_pages() {
	for DOCS in man info; do
		if [ -d "$startdir/pkg/usr/share/$DOCS" ]; then
			mv $startdir/pkg/usr/share/$DOCS $startdir/pkg/usr/$DOCS
			if [[ ! "$(ls $startdir/pkg/usr/share)" ]]; then
				rm -rf $startdir/pkg/usr/share
			fi
		fi
		if [ -d "$startdir/pkg/usr/$DOCS" ]; then
			# I've never seen symlinks in info pages....
			if [ "$DOCS" == "man" ]; then
				(cd $startdir/pkg/usr/$DOCS
				for manpagedir in $(find . -type d -name "man*" 2> /dev/null) ; do
					( cd $manpagedir
					for eachpage in $( find . -type l -maxdepth 1 2> /dev/null) ; do
						ln -s $( readlink $eachpage ).gz $eachpage.gz
						rm $eachpage
					done )
				done)
			fi
			find $startdir/pkg/usr/$DOCS -type f -exec gzip -9 '{}' \;
		fi
	done
}

set_post_permissions() {
	for DIRS in usr/share/icons usr/doc; do
		if [ -d "$startdir/pkg/$DIRS" ]; then
			if [ "$DIRS" == "usr/doc" ]; then
				find $startdir/pkg/$DIRS -type f -exec chmod 644 {} \;
				find $startdir/pkg/$DIRS -type d -exec chmod 755 {} \;
			fi
		fi
		[ -d $startdir/pkg/$DIRS ] && chown root:root -R $startdir/pkg/$DIRS
	done
	[ -d $startdir/pkg/usr/bin ] && find $startdir/pkg/usr/bin -user root -group bin -exec chown root:root {} \;
}

copy_build_script() {
	mkdir -p $startdir/pkg/usr/src/$pkgname-$pkgver/
	cp $startdir/build-$pkgname.sh $startdir/pkg/usr/src/$pkgname-$pkgver/build-$pkgname.sh
	[ -f $startdir/ZENBUILD ] && cp $startdir/ZENBUILD $startdir/pkg/usr/src/$pkgname-$pkgver/ZENBUILD
}

create_package() {
	ls -lR $startdir/pkg
	cd $startdir/pkg
	/sbin/makepkg -l y -c n $startdir/$package.txz
	cd $startdir
	md5sum $package.txz > $startdir/$package.md5
}

strip_binaries() {
	cd $startdir/pkg
	find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | \
	xargs strip --strip-unneeded 2> /dev/null
	find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | \
	xargs strip --strip-unneeded 2> /dev/null
}
#########End Redundant Code#####################################
#########Begin Non Redundant Code##############################

prepare_directory() {
	mkdir $startdir/src
	mkdir -p $startdir/pkg/usr/src/$pkgname-$pkgver
	for SOURCES in ${source[@]}; do
		protocol=$(echo $SOURCES | sed 's|:.*||')
	        file=$(basename $SOURCES | awk -F= '{print $NF}')
		if [ ! -f "$file" ]; then
			if [ "$protocol" = "http" -o "$protocol" = "https" -o "$protocol" = "ftp" ]; then
				echo -e "\nDownloading $(basename $SOURCES)\n"
        	                wget $SOURCES -O $file
				if [ ! "$?" == "0" ]; then
					echo "Download failed"
					exit 2
				fi 
			else
				echo "$SOURCES does not appear to be a url nor is it in the directory"
				exit 2
			fi
		fi
		cp -R $file $startdir/src
		if ! [ "$protocol" = "http" -o "$protocol" = "https" -o "$protocol" = "ftp" ]; then
			cp -R $startdir/$(basename $SOURCES) $startdir/pkg/usr/src/$pkgname-$pkgver/
		fi
		if [ "${SOURCES##*.}" == "desktop" ]; then
			[[ "$(grep -e "^Categories=[a-zA-Z]*;" $(basename $SOURCES))" ]] && install -D $startdir/$(basename $SOURCES) /home/shuriken/ZUR/truecrypt/pkg/usr/share/applications/$(basename $SOURCES)
		fi
	done
}

extract_source() {
	cd $startdir/src
	if [[ "$(ls $startdir/src)" ]]; then	
		for FILES in ${source[@]}; do
	        	FILES="$(basename $FILES | awk -F= '{print $NF}')"
			file_type=$(file -biz "$FILES")
			unset cmd
			case "$file_type" in
				*application/x-tar*)
					cmd="tar -xf" ;;
				*application/x-zip*)
					cmd="unzip" ;;
				*application/x-gzip*)
					cmd="gunzip -d -f" ;;
				*application/x-bzip*)
					cmd="bunzip2 -f" ;;
			esac
			if [ "$cmd" != "" ]; then
				echo "$cmd $FILES"
	        	        $cmd $FILES
			fi
		done
	elif [ ! "$source" ]; then
		echo -n "" # lame fix
	else
		echo "no files in the src directory $startdir/src"
		exit 2
	fi
}

build() {

##Make necessary directories

	mkdir -p $startdir/pkg/usr/bin
	mkdir -p $startdir/pkg/usr/share/applications
	mkdir -p $startdir/pkg/usr/share/icons/hicolor/{16x16,32x32,48x48}/apps
	mkdir -p $startdir/pkg/usr/doc

##Binary build process

	cd $startdir/src/$pkgname-$pkgver-source
	make -j2 \
		NOSTRIP=1 VERBOSE=1 NOTEST=1 EXTRA_CFLAGS="-O2 -march=i686 -mtune=i686" \
		EXTRA_CXXFLAGS="-O2 -march=i686 -mtune=i686 -DCKR_NEXT_OTP=0x000001B -DCKR_NEW_PIN_MODE=0x000001B0" \
		EXTRA_LDFLAGS=$LDFLAGS WX_CONFIG="/usr/bin/wx-config" PKCS11_INC="$startdir/src/" 
	install -m0755 Main/$pkgname $startdir/pkg/usr/bin/ || exit 1
	
##Create desktop file

	cat > $startdir/pkg/usr/share/applications/$pkgname.desktop <<EOF
[Desktop Entry]
Name=TrueCrypt
Comment=Disk encryption software
Exec=$pkgname
Icon=$pkgname
Terminal=false
Type=Application
Categories=GTK;System;
X-Desktop-File-Install-Version=0.15
EOF

##Create Icons

	convert Common/TrueCrypt.ico -delete 0,1,2,4 -depth 32 \
        $startdir/pkg/usr/share/icons/hicolor/16x16/apps/$pkgname.png || exit 1
	convert Common/TrueCrypt.ico -delete 0,1,2,4 -depth 32 \
        $startdir/pkg/usr/share/icons/hicolor/32x32/apps/$pkgname.png || exit 1
	convert Common/TrueCrypt.ico -delete 0,1,2,4 -depth 32 \
        $startdir/pkg/usr/share/icons/hicolor/48x48/apps/$pkgname.png || exit 1
		
##Rename PDF

	cd Release/Setup*Files/ 
	mv *.pdf TrueCrypt_User_Guide.pdf
} 

create_slackdesc() {
mkdir $startdir/pkg/install
cat <<"EODESC" >$startdir/pkg/install/slack-desc
truecrypt: TrueCrypt (Free Open-Source On-The-Fly Encryption)
truecrypt: 
truecrypt: Free open-source disk encryption software 
truecrypt: for Windows Vista/XP, Mac OS X, and Linux.
truecrypt: 
truecrypt: 
truecrypt: 
truecrypt: 
truecrypt: 
truecrypt: 
truecrypt: 
EODESC
}

setup_doinst() {
cat <<"EODOINST" >>$startdir/pkg/install/doinst.sh
doinst() {
## Check for fsck.vfat, if none, create one
fsck_vfat_check()
	{	
	if [ ! -r /sbin/fsck.vfat ]; then
		ln -s /sbin/dosfsck /sbin/fsck.vfat
	fi
	}
## Create Sudo Rule
make_sudoers_temp()
	{
	cp /etc/sudoers /etc/sudoers.old
	cp /etc/sudoers /etc/sudoers.copy
	chmod 777 /etc/sudoers.copy
	}
add_tc_sudo_rule()
	{
	cat >> /etc/sudoers.copy <<EOF

##TrueCrypt Rules
%users ALL = (root) /usr/bin/truecrypt --core-service
EOF
	}
save_sudoers_mods()
	{
	cp /etc/sudoers.copy /etc/sudoers
	chmod 0440 /etc/sudoers
	}
create_sudoers()
	{
	cp /etc/sudoers.new /etc/sudoers
	}

##Add dm-crypt mappings removal code to /etc/rc.d/rc.6
tc_rm_map()
	{
	if ! grep -q "truecrypt -l" /etc/rc.d/rc.6; then
	perl -pi -le 'print "" if $. == 176' /etc/rc.d/rc.6
	perl -pi -le 'print "  # Try to remove any dm-crypt mappings (for TrueCrypt)" if $. == 177' /etc/rc.d/rc.6
	perl -pi -le 'print "  if [ -x /usr/bin/truecrypt ]; then" if $. == 178' /etc/rc.d/rc.6
	perl -pi -le 'print "  echo \"Removing TrueCrypt mappings\"" if $. == 179' /etc/rc.d/rc.6
	perl -pi -le 'print "    ! /usr/bin/truecrypt -l > /dev/null 2>&1  || /usr/bin/truecrypt -d" if $. == 180' /etc/rc.d/rc.6
	perl -pi -le 'print "  fi" if $. == 181' /etc/rc.d/rc.6
	fi
	}

	
fsck_vfat_check
if [ ! -r /etc/sudoers ]; then
	create_sudoers
	make_sudoers_temp
	add_tc_sudo_rule
	save_sudoers_mods
	rm /etc/sudoers.copy
else
	make_sudoers_temp
	if ! grep -q "%users ALL = (root) /usr/bin/truecrypt --core-service" /etc/sudoers.copy; then
		add_tc_sudo_rule
		save_sudoers_mods
		rm /etc/sudoers.copy
	else
		rm /etc/sudoers.copy
	fi	
fi
tc_rm_map

exit 0
}
doinst
EODOINST
}
copy_docs() {
	for stuff in ${docs[@]}; do
		if [ ! -d "$startdir/pkg/usr/doc/$pkgname-$pkgver" ]; then
			mkdir -p $startdir/pkg/usr/doc/$pkgname-$pkgver
		fi
		find $startdir/src -type f -iname "$stuff" -exec cp -R '{}' $startdir/pkg/usr/doc/$pkgname-$pkgver \;
	done
}
detect_dependencies() {
	echo "Detecting dependencies..."
	depends=$(requiredbuilder -z $startdir/$package.txz)
	
	depstring=$depends,sudo
	unset FIRST
	for deps in $(echo "${depstring}" | sed 's/,/\n/g' | sort -u); do
		if [[ ! "$FIRST" ]]; then
			finaldeps=$deps,
			FIRST='1'
			continue
		fi
		finaldeps=${finaldeps}${deps},
	done
	echo "$finaldeps" | sed -e 's/^,//' -e 's/,$//' > $startdir/$package.dep
}
create_source_file(){
	[ -f $package.src ] && rm $package.src
	if [ ! -z $sourcetemplate ]; then
		echo $sourcetemplate/ZENBUILD >> $package.src
		echo $sourcetemplate/build-$pkgname.sh >> $package.src
	else
		echo ZENBUILD >> $package.src
		echo build-$pkgname.sh >> $package.src
	fi
	for SOURCES in ${source[@]}; do
		protocol=$(echo $SOURCES | sed 's|:.*||')
		if ! [ "$protocol" = "http" -o "$protocol" = "https" -o "$protocol" = "ftp" ]; then
			if [ ! -z $sourcetemplate ]; then
				echo $sourcetemplate/$(basename $SOURCES) >> $package.src
			else
				echo $(basename $SOURCES) >> $package.src
			fi
		else
			echo $SOURCES >> $package.src
		fi
	done
}
post_checks(){
	# Ideas taken from src2pkg :)
	if [ -d "$startdir/pkg/usr/doc/$pkgname-$pkgver" ]; then
		for DIRS in usr/doc/$pkgname-$pkgver usr/doc; do
			cd $startdir/pkg/$DIRS
			if [[ $(find . -type f) = "" ]] ; then
				cd ..
				rmdir $DIRS
			fi
		done
	fi
	# if the docs weren't deleted ...
	if [ -d "$startdir/pkg/usr/doc/$pkgname-$pkgver" ]; then
		cd $startdir/pkg/usr/doc/$pkgname-$pkgver
		#remove zero lenght files
		if [[ $(find . -type f -size 0) ]]; then
			echo "Removing some zero lenght files"
			find . -type f -size 0 -exec rm -f {} \;
		fi
	fi
	# check if we need to add code to handle info pages
	if [[ -d $startdir/pkg/usr/info ]] && [[ ! $(grep install-info $startdir/pkg/install/doinst.sh &> /dev/null) ]] ; then
		echo "Found info files - Adding install-info command to doinst.sh"
		INFO_LIST=$(ls -1 $startdir/pkg/usr/info)
		echo "" >> $startdir/pkg/install/doinst.sh
		echo "if [ -x usr/bin/install-info ] ; then" >> $startdir/pkg/install/doinst.sh
		for page in $(echo $INFO_LIST) ; do
			echo " usr/bin/install-info --info-dir=usr/info usr/info/$page 2>/dev/null" >> $startdir/pkg/install/doinst.sh
		done
		echo "fi" >> $startdir/pkg/install/doinst.sh
	fi
	[[ -e $startdir/pkg/usr/info/dir ]] && rm -f $startdir/pkg/usr/info/dir

	if [ -d $startdir/pkg/etc ]; then
		cd $startdir/pkg/
		for conf in $(find ./etc -type f) ; do
			conf=${conf: 2}
			dotnew=( "${dotnew[@]}" "$conf" )
		done
	fi
	if [[ "$dotnew" ]]; then
        for files in ${dotnew[@]} ; do
                fullfile="${startdir}/pkg/${files}"
                if [ -e "$fullfile" ]; then
                        mv $fullfile ${fullfile}.new
                else
                        echo "$fullfile was not found"
                        exit 2
                fi
        done
        cat<<"EODOTNEW" >>$startdir/pkg/install/doinst.sh
#Added by buildpkg 1.6.1
dotnew() {
        NEW="${1}.new"
        OLD="$1"
        if [ ! -e $OLD ]; then
                mv $NEW $OLD
        elif [ "$(cat $OLD | md5sum)" = "$(cat $NEW | md5sum)" ]; then
                rm $NEW
        fi
}
EODOTNEW
for i in ${dotnew[@]}; do
echo "dotnew $i" >> $startdir/pkg/install/doinst.sh
done
fi
}

####End Non Redundant Code############################

#Variables

startdir=$(pwd)
SRC=$startdir/src
PKG=$startdir/pkg

pkgname=truecrypt
pkgver=6.3a
pkgrel=1
zenver=64
arch=i686
package=$pkgname-$pkgver-$arch-$zenver.$pkgrel
source=("http://zenwalk.pinguix.com/user-accounts/shingoku/truecrypt/source/TrueCrypt_6.3a_Source.tar.gz" "http://zenwalk.pinguix.com/user-accounts/shingoku/truecrypt/source/pkcs11/cryptoki.h" "http://zenwalk.pinguix.com/user-accounts/shingoku/truecrypt/source/pkcs11/ct-kip.h" "http://zenwalk.pinguix.com/user-accounts/shingoku/truecrypt/source/pkcs11/otp-pkcs11.h" "http://zenwalk.pinguix.com/user-accounts/shingoku/truecrypt/source/pkcs11/pkcs-11v2-20a3.h" "http://zenwalk.pinguix.com/user-accounts/shingoku/truecrypt/source/pkcs11/pkcs-11v2-20a3d3.h" "http://zenwalk.pinguix.com/user-accounts/shingoku/truecrypt/source/pkcs11/pkcs11.h" "http://zenwalk.pinguix.com/user-accounts/shingoku/truecrypt/source/pkcs11/pkcs11f.h" "http://zenwalk.pinguix.com/user-accounts/shingoku/truecrypt/source/pkcs11/pkcs11t-consolidated-d1.h" "http://zenwalk.pinguix.com/user-accounts/shingoku/truecrypt/source/pkcs11/pkcs11t.h" "http://zenwalk.pinguix.com/user-accounts/shingoku/truecrypt/source/pkcs11/pkcs11t.h.org")
sourcetemplate=http://zenwalk.pinguix.com/user-accounts/shingoku/truecrypt/source
docs=(Readme.txt License.txt TrueCrypt_User_Guide.pdf)
export CFLAGS="-O2 -march=i686 -mtune=i686"
export CXXFLAGS="-O2 -march=i686 -mtune=i686"

#Execution

check_for_root
clean_old_builds
prepare_directory
extract_source
set_pre_permissions
build
if [ ! "$?" = "0" ]; then
	echo "build() failed."
	exit 2
fi
create_slackdesc
post_checks
setup_doinst
copy_docs
check_for_menu_compliance
strip_binaries
gzip_man_and_info_pages
set_post_permissions
copy_build_script
create_package
create_source_file
detect_dependencies
echo "Package has been built."
echo "Cleaning pkg and src directories"
clean_dirs
